#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# OBJ - timestamped JSON objects
#
# bin/obj
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 16-02-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" command line interface. """

## imports

from obj import Object
from obj.core import fleet, loader
from obj.handler import Handler

import logging
import sys

## init

def start():
    bot = CLI()
    bot.start()
    return bot

## classes

class CLI(Handler):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.prompted = False

    def cmd(self, txt):
        """ execute a command based on txt.  """
        from obj.core import start
        name = txt.split()[0]
        logging.warn("cmd %s" % name)
        event = self.event(txt)
        event.origin = "root@shell"
        event.handle()
        return event
        
    def event(self, txt=""):
        """ must return an Event and block for input function. """ 
        e = Object()
        e._loader = loader
        e.id = self.id
        e.cc = self.cc
        e.origin = "root@shell"
        e.txt = txt or input()
        self.prompted = False
        e.txt = e.txt.strip()
        return e

    def handle(self, event):
        super().handle(event)
        self.prompt()

    def prompt(self, *args, **kwargs):
        """ echo prompt to sys.stdout. """
        if self.prompted:
            return
        self.prompted = True
        sys.stdout.write("> ")
        sys.stdout.flush()

    def raw(self, txt):
        """ output to console. """
        print(txt)

    def say(self, channel, txt, type="normal"):
        """ overloaded say method. """
        self.raw(txt)    

    def start(self):
        from obj import cfg
        fleet.append(self)
        loader.walk("obj.cmds")
        if cfg.args:
            return self.cmd(" ".join(cfg.args))
        elif cfg.shell:
            super().start()
            self.prompt()
            self.wait()
        self.ready()
